@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 import logging
+from re import I
 from typing import Any, Callable, Optional
 
 import ckan.plugins.toolkit as tk
@@ -11,7 +12,8 @@ from ckan.logic import validate
 from ckanext.transmute.types import TransmuteData, Field
 from ckanext.transmute.schema import SchemaParser, SchemaField
 from ckanext.transmute.schema import transmute_schema, validate_schema
-from ckanext.transmute.exception import ValidationError
+from ckanext.transmute.exception import ValidationError, TransmutatorError
+from ckanext.transmute.utils import get_transmutator
 
 
 log = logging.getLogger(__name__)
@@ -98,7 +100,15 @@ def mutate_old_fields(data, definition, root):
             data[field_name] = value = data[field.get_replace_from()]
 
         if field.value:
-            data[field_name] = value = field.value
+            if field.update:
+                if isinstance(data[field_name], dict):
+                    data[field_name].update(field.value)
+                elif isinstance(data[field_name], list):
+                    data[field_name].extend(field.value)
+                else:
+                    raise ValidationError({f"{field_name}: the field value is immutable"})
+            else:
+                data[field_name] = value = field.value
 
         if field.is_multiple():
             for nested_field in value:
@@ -165,14 +175,11 @@ def _apply_validators(field: Field, validators: list[Callable[[Field], Any]]):
     try:
         for validator in validators:
             if isinstance(validator, list):
-
-                # TODO show some error if there are less then 1 arg
                 if len(validator) <= 1:
-                    pass
-                
-                tk.get_validator(validator[0])(field, *validator[1:])
+                    raise TransmutatorError("Arguments for validator weren't provided")
+                field = get_transmutator(validator[0])(field, *validator[1:])
             else:
-                field = tk.get_validator(validator)(field)
+                field = get_transmutator(validator)(field)
     except df.Invalid as e:
         raise ValidationError({f"{field.type}:{field.field_name}": [e.error]})
 
